/********************************************************************************
 *                 -            *
 *              NVCom-01MEM-3U (SST39VF640)                                     *
 *                                                                              *
 * Last correction: 03/December/2012                                            *
 *                                                                              *
 *  1.                                                      *
 *  2.       1 (    =0)*
 *      -   start_val=0           *
 *     -   base_addr=0xbf000000             *
 *    -   flash_size=16                         *
 *           mdb  main:    *
 *    bp 0 main                                                                 *
 *    run                                                                       *
 *    set start_val 0x1234                                                      *
 *    set base_addr 0xbfc00000                                                  *
 *    set flash_size 0x00800000                                                 *
 *    set uart_num 1                                                            *
 *    run                                                                       *
 *  3.                                               *
 ********************************************************************************
*/

#include <stdio.h>
#include <stdarg.h>
#include "flash_test.h"

#define FLASH_SIZE    (16*1024*1024)

FLASH_DESCRIPTOR fd;

unsigned int get_cpu_clock(void);
unsigned int get_mport_clock(void);
void init_uart(unsigned int n);
void uart_ch(char c);
void uart_str(char* s);
void uart_conf(unsigned int speed,unsigned int freq);
void clear_screen(void);
void print_console(const char *fmt,...);
void cache_on(void);

unsigned int freq;
unsigned int freq_mport;
unsigned int uart_num=0;

extern void init_flash_descriptor(FLASH_DESCRIPTOR* p,unsigned int base);
extern unsigned int get_man_id(FLASH_DESCRIPTOR* p);
extern unsigned int get_device_id(FLASH_DESCRIPTOR* p);
extern void flash_write_word(FLASH_DESCRIPTOR* p,unsigned int a,unsigned int v);
extern void chip_erase(FLASH_DESCRIPTOR* p);
extern void dummy_read(void);

inline void zero_count(void);
inline unsigned int get_count(void);
void stop_counter(void);
void flash_write_buf(unsigned int start_addr,unsigned int *p,unsigned int n);
void cache_on(void);

unsigned int start_val=0;
unsigned int base_addr=BASE_ADDR;
unsigned int flash_size=FLASH_SIZE;

unsigned int bpt=0;
unsigned int bev=0;
unsigned int volatile   val=0;
unsigned int volatile irq_cnt=0;
unsigned int volatile last_irq_cnt=0;
unsigned int volatile max_count=0;
unsigned int volatile   tick_sec;
unsigned int volatile   last_sec=0,sec=0;
unsigned int write_sec;
unsigned int verify_sec;
unsigned int man_id=0;
unsigned int dev_id=0;
unsigned int nerr=0;

int main(void)
{
    unsigned int i,n,*p,len,val;


    init_flash_descriptor(&fd,BASE_ADDR);

    man_id=get_man_id(&fd);
    dev_id=get_device_id(&fd);

    chip_erase(&fd);
    fd.base=base_addr;
    len=flash_size;
    for (n=0,i=start_val;n<len;n+=4,i++) {
        flash_write_word(&fd,n,i);
    };

    nerr=0;
    for (p=(unsigned int*)fd.base,n=0,i=start_val;n<len;n+=4,i++) {
        val=*p++;
        if (val!=i) nerr++;
    };

    while (1);
};

/**
 *
 */
void flash_write_buf(unsigned int start_addr,unsigned int *p,unsigned int n)
{
    unsigned int i,addr;

    for (i=0,addr=start_addr-BASE_ADDR;i<n;i++,addr+=4) {
        flash_write_word(&fd,addr,p[i]);
    };
};

/**
 *    (CPU)
 * @return -    Hz
 */
unsigned int get_cpu_clock(void)
{
    return((CLOCK)*(SYSTEM->CR_PLL&0xff));
};

/**
 *     
 * @return -      Hz
 */
unsigned int get_mport_clock(void)
{
    return((CLOCK)*((SYSTEM->CR_PLL>>8)&0xff));
};
