/**
 * Copyright (C) 2012 by Elvees
 *
 * Created:         03/December/2012
 * Last correction: 03/December/2012
 *
 */

#include <stdio.h>
#include <stdarg.h>
#include "flash_test.h"

void init_flash_descriptor(FLASH_DESCRIPTOR* p,unsigned int base);
unsigned int flash_read(FLASH_DESCRIPTOR* p,unsigned int a);
void flash_write(FLASH_DESCRIPTOR* p,unsigned int a,unsigned int v);
unsigned int get_flash_cmd(FLASH_DESCRIPTOR* p,FLASH_CMD cmd);
void flash_cmd(FLASH_DESCRIPTOR* p,FLASH_CMD cmd);
void exit_cfi(FLASH_DESCRIPTOR* p);
unsigned int get_cfi(FLASH_DESCRIPTOR* p,unsigned int addr);
unsigned int get_man_id(FLASH_DESCRIPTOR* p);
unsigned int get_device_id(FLASH_DESCRIPTOR* p);
void unlock_blk(FLASH_DESCRIPTOR* p,unsigned int a);
void chip_unlock(FLASH_DESCRIPTOR* p);
void wait_wsms(FLASH_DESCRIPTOR* p);
void chip_erase(FLASH_DESCRIPTOR* p);
void sector_erase(FLASH_DESCRIPTOR* p,unsigned int a);
void block_erase(FLASH_DESCRIPTOR* p,unsigned int a);
void flash_write_word(FLASH_DESCRIPTOR* p,unsigned int a,unsigned int v);
unsigned int flash_empty(FLASH_DESCRIPTOR* p,unsigned int len);
unsigned int blk_empty(FLASH_DESCRIPTOR* p,unsigned int a,unsigned int len);
void dummy_read(void);
extern void print_console(const char *fmt, ...);

void init_flash_descriptor(FLASH_DESCRIPTOR* p,unsigned int base)
{
    p->type=0;
    p->base=base;  /*    - */

    p->addr_odd=0x5555<<2;
    p->addr_even=0x2aaa<<2;

    p->cmd_10=FLASH16(0x10);
    p->cmd_20=FLASH16(0x20);
    p->cmd_30=FLASH16(0x30);
    p->cmd_40=FLASH16(0x40);
    p->cmd_50=FLASH16(0x50);
    p->cmd_55=FLASH16(0x55);
    p->cmd_60=FLASH16(0x60);
    p->cmd_80=FLASH16(0x80);
    p->cmd_88=FLASH16(0x88);
    p->cmd_90=FLASH16(0x90);
    p->cmd_98=FLASH16(0x98);
    p->cmd_a0=FLASH16(0xa0);
    p->cmd_aa=FLASH16(0xaa);
    p->cmd_d0=FLASH16(0xd0);
    p->cmd_f0=FLASH16(0xf0);
    p->cmd_ff=FLASH16(0xff);
};

/**
 *
 */
unsigned int flash_read(FLASH_DESCRIPTOR* p,unsigned int a)
{
    unsigned int v,addr=p->base+a;

    v=*(volatile unsigned int*)addr;
    return(v);
};

/**
 *
 */
void flash_write(FLASH_DESCRIPTOR* p,unsigned int a,unsigned int v)
{
    unsigned int addr=p->base+a;
    *(volatile unsigned int*)addr=v;
};

/**
 *
 */
unsigned int get_flash_cmd(FLASH_DESCRIPTOR* p,FLASH_CMD cmd)
{
    switch (cmd) {
        case CMD_10: return(p->cmd_10);
        case CMD_20: return(p->cmd_20);
        case CMD_30: return(p->cmd_30);
        case CMD_40: return(p->cmd_40);
        case CMD_50: return(p->cmd_50);
        case CMD_55: return(p->cmd_55);
        case CMD_60: return(p->cmd_60);
        case CMD_80: return(p->cmd_80);
        case CMD_88: return(p->cmd_88);
        case CMD_90: return(p->cmd_90);
        case CMD_98: return(p->cmd_98);
        case CMD_A0: return(p->cmd_a0);
        case CMD_AA: return(p->cmd_aa);
        case CMD_D0: return(p->cmd_d0);
        case CMD_F0: return(p->cmd_f0);
        case CMD_FF: return(p->cmd_ff);
    };
    return(0);
};

/**
 *
 */
void flash_cmd(FLASH_DESCRIPTOR* p,FLASH_CMD cmd)
{
    if (!p->type) {
        flash_write(p,p->addr_odd,p->cmd_aa);
        flash_write(p,p->addr_even,p->cmd_55);
        flash_write(p,p->addr_odd,get_flash_cmd(p,cmd));
    } else {
        flash_write(p,0,get_flash_cmd(p,cmd));
    };
};

/**
 *
 */
void exit_cfi(FLASH_DESCRIPTOR* p)
{
    flash_cmd(p,CMD_F0);
};

/**
 *
 */
unsigned int get_cfi(FLASH_DESCRIPTOR* p,unsigned int addr)
{
    unsigned int val;

    flash_cmd(p,CMD_98);
    val=flash_read(p,ADR32(addr));
    exit_cfi(p);
    return(val&0xffff);
};

/**
 * Get Manufacturer's ID
 */
unsigned int get_man_id(FLASH_DESCRIPTOR* p)
{
    return(get_cfi(p,MAN_ID_OFFSET));
};

/**
 * Get Device ID
 */
unsigned int get_device_id(FLASH_DESCRIPTOR* p)
{
    return(get_cfi(p,DEVICE_ID_OFFSET));
};

/**
 *
 */
void wait_wsms(FLASH_DESCRIPTOR* p)
{
    volatile unsigned int v;

    do {
        v=flash_read(p,0);
    } while ((v&WSMS)!=WSMS);
};


/**
 * Unlock block
 */
void unlock_blk(FLASH_DESCRIPTOR* p,unsigned int a)
{
    if (p->type) {
        flash_write(p,a,get_flash_cmd(p,CMD_60));
        flash_write(p,a,get_flash_cmd(p,CMD_D0));
        flash_cmd(p,CMD_FF);
        exit_cfi(p);
    };
};

/**
 * Unlock all chip
 */
void chip_unlock(FLASH_DESCRIPTOR* p)
{
    unsigned int a,blk1,blk2,step1,step2;

    if (p->type) {
        blk1=8; step1=0x4000;
        for (a=0;a<(blk1*step1);a+=step1) {
            unlock_blk(p,a);
        };
        blk2=64; step2=0x20000;
        for (a=blk1*step1;a<(blk2*step2);a+=step2) {
            unlock_blk(p,a);
        };
    };
};


/**
 * Chip Erase
 */
void chip_erase(FLASH_DESCRIPTOR* p)
{
    unsigned int i,val1,val2,a,blk1,blk2,step1,step2;

    if (!p->type) {
        flash_cmd(p,CMD_80);
        flash_cmd(p,CMD_10);
        i=0;
        do {
            val1=flash_read(p,0);
            dummy_read();
            val2=flash_read(p,0);
        } while (val1!=val2);
    } else {
        blk1=8; step1=0x4000;
        for (a=0;a<(blk1*step1);a+=step1) {
            if (blk_empty(p,a,step1)) {
                unlock_blk(p,a);
                sector_erase(p,a);
            };
        };
        blk2=64; step2=0x20000;
        for (a=blk1*step1;a<(blk2*step2);a+=step2) {
            if (blk_empty(p,a,step2)) {
                unlock_blk(p,a);
                sector_erase(p,a);
            };
        };
    };
};

/**
 * Sector Erase
 */
void sector_erase(FLASH_DESCRIPTOR* p,unsigned int a)
{
    unsigned int i,val1,val2;

    if (!p->type) {
        flash_cmd(p,CMD_80);
        flash_write(p,p->addr_odd,p->cmd_aa);
        flash_write(p,p->addr_even,p->cmd_55);
        flash_write(p,a,get_flash_cmd(p,CMD_50));
        i=0;
        do {
            val1=flash_read(p,8);
            dummy_read();
            val2=flash_read(p,8);
        } while (val1!=val2);
        exit_cfi(p);
    } else {
        unlock_blk(p,a);
        flash_cmd(p,CMD_20);
        flash_write(p,a,get_flash_cmd(p,CMD_D0));
        wait_wsms(p);
        flash_cmd(p,CMD_50);
        flash_cmd(p,CMD_FF);
        exit_cfi(p);
    };
};

/**
 * Block Erase
 */
void block_erase(FLASH_DESCRIPTOR* p,unsigned int a)
{
    unsigned int i,val1,val2;

    flash_cmd(p,CMD_80);
    flash_write(p,p->addr_odd,p->cmd_aa);
    flash_write(p,p->addr_even,p->cmd_55);
    flash_write(p,a,get_flash_cmd(p,CMD_30));
    i=0;
    do {
        val1=flash_read(p,8);
        dummy_read();
        val2=flash_read(p,8);
    } while (val1!=val2);
};


/**
 * Write word to flash
 */
void flash_write_word(FLASH_DESCRIPTOR* p,unsigned int a,unsigned int v)
{
    unsigned int i,val1,val2;

    if (!p->type) {
        flash_cmd(p,CMD_A0);
        flash_write(p,a,v);
        i=0;
        do {
            val1=flash_read(p,8);
            dummy_read();
            val2=flash_read(p,8);
        } while (val1!=val2);
    } else {
        flash_cmd(p,CMD_50);
        flash_cmd(p,CMD_40);
        flash_write(p,a,v);
        wait_wsms(p);
        flash_cmd(p,CMD_50);
        flash_cmd(p,CMD_FF);
    };
};


/**
 *  -  ?
 */
#define BLK     0x10000
unsigned int flash_empty(FLASH_DESCRIPTOR* p,unsigned int len)
{
#if 1
    unsigned int i,base,*q;

    base=p->base;
    q=(unsigned int*)base;
#else
    r = malloc(len);
	if (r == NULL) {
		perror("malloc");
		return(-1);
	};
    mem_read(r,base,len);
    len>>=2;
    for (i=0;i<len;i++) {
        if (i&&((i%BLK)==0))
            printf("\r0x%08x",&q[i]);
        if (r[i]!=0xffffffff) {
            printf("\n[0x%08x]=0x%08x\n",(unsigned int)(&q[i]),r[i]);
            return((unsigned int)(&q[i]));
        };
    };
    free(r);
#endif
    len>>=2;
    for (i=0;i<len;i++) {
        if (q[i]!=0xffffffff) {
            return((unsigned int)(&q[i]));
        };
    };
    return(0);
};

/**
 *
 */
unsigned int blk_empty(FLASH_DESCRIPTOR* p,unsigned int a,unsigned int len)
{
    unsigned int i,*q;

    len>>=2;
    q=(unsigned int*)(p->base+a);
    for (i=0;i<len;i++) {
        if (q[i]!=0xffffffff) {
            return((unsigned int)(&q[i]));
        };
    };
    return(0);
};

/**
 *     ,   MPort
 */
void dummy_read(void)
{
    unsigned int v;
    v=*(volatile unsigned int*)(ADDR_DUMMY);
};


