/**
 * Copyright (C) 2012 by Elvees
 *
 * Created:         03/December/2012
 * Last correction: 03/December/2012
 *
 */
#ifndef _FLASH_TEST_H_
#define _FLASH_TEST_H_

#define C0_COUNT	    9	                /*  */
#define C0_COMPARE	    11	                /*       */
#define C0_STATUS	    12	                /*    CP0:    */
#define C0_CAUSE	    13	                /*    */
#define ST_IE		    (1<<0)              /*    */
#define ST_IM_WDT       (1<<10)             /*    QSTR0,    */
#define ST_BEV		    (1<<22)             /*   :   */

#define ST_IM_COMPARE   (1<<15)

/**
 * Read C0 coprocessor register.
 */
#define mips_read_c0_register(reg)				\
({ int __value;							\
	asm volatile (						\
	"mfc0	%0, $%1"					\
	: "=r" (__value) : "K" (reg));				\
	__value;						\
})

/**
 * Write C0 coprocessor register.
 */
#define mips_write_c0_register(reg, value)			\
	do {							\
	asm volatile (						\
	"mtc0	%z0, $%1 \n	nop \n	nop \n	nop"		\
	: : "r" ((unsigned int) (value)), "K" (reg));		\
	} while (0)

#define UART_SPEED      115200              /*   (/) */

#define BASE_REG        0xb82f              /*  CPU (   ) */
#define ADDR(hi,lo)     (hi<<16|lo)
#define REG(lo)         ADDR(BASE_REG,lo)   /*   CPU */

/**
 *      BEV = 0:
 * 0        (0x80000000)
 * 1        CRAM (0xb8000000)
 */
#define TR_CRAM         (1<<1)

#define MHz             1000000
#define CLOCK           5*MHz      /*   - 10/2 MHz */

/* software breakpoint */
#define STOP            asm volatile("nop"); \
                        asm volatile("nop"); \
                        asm volatile("nop"); \
                        asm volatile("nop"); \
                        asm volatile(".word 0x4d"); \
                        asm volatile("nop"); \
                        asm volatile("nop"); \
                        asm volatile("nop")

typedef volatile struct {
    unsigned int CR_PLL;    /*   PLL (+0x00) */
    unsigned int CLK_EN;    /*       (+0x04) */
    unsigned int CSR;       /*   (+0x08) */
    unsigned int reserved_0c; /*+0x0c */
    union {
        unsigned int MASKR; /*   (+0x10) */
        unsigned int MASKR0;/*   (+0x10) */
    };
    union {
        unsigned int QSTR;  /*   (+0x14) */
        unsigned int QSTR0; /*   (+0x14) */
    };
    unsigned int MASKR1;    /*   (+0x18) */
    unsigned int QSTR1;     /*   (+0x1c) */
    unsigned int MASKR2;    /*   (+0x20) */
    unsigned int QSTR2;     /*   (+0x24) */
    unsigned int reserved_28; /*+0x28 */
    unsigned int reserved_2c; /*+0x2c */
    unsigned int IRQM;      /*       nIRQ[3:0] (+0x30) */
} _SYSTEM;

#define SYSTEM  ((_SYSTEM*)REG(0x4000))

/**
 * UART - Universal Asynchronous Receiver/Transmitter
 *  unnamed struct/union fields within structs/unions
 *        union.
 */
typedef volatile struct {   /*      */
    union {
        unsigned int RBR;   /* +0x00    8           R          (Receiver Buffer Register) */
        unsigned int THR;   /* +0x00    8           W          (Transmitter Holding Register) */
        unsigned int DLL;   /* +0x00    8           RW         (DLAB=1)(Divisor Latch register Least significant byte) */
    };
    union {
        unsigned int IER;   /* +0x04    8           RW         (Interrupt Identification Register) */
        unsigned int DLM;   /* +0x04    8           RW         (DLAB=1) (Divisor Latch register Most significant byte) */
    };
    union {
        unsigned int IIR;   /* +0x08    8           R          (Interrupt Identification Register) */
        unsigned int FCR;   /* +0x08    8           W         FIFO (FIFO Control Register) */
    };
    unsigned int LCR;       /* +0x0c    8           RW         (Line Control Register) */
    unsigned int MCR;       /* +0x10    8           RW         (Modem Control Register) */
    union {
        unsigned int LSR;   /* +0x14    8           R          (Line Status Register) */
        unsigned int SCLR;  /* +0x14    8           W         (scaler) */
    };
    unsigned int MSR;       /* +0x18    8           RW         (Modem Status Register) */
    unsigned int SPR;       /* +0x1c    8           RW       Scratch Pad (Scratch Pad Register) */
} _UART;

#define UART0    ((_UART*)REG(0x3000))
#define UART1    ((_UART*)REG(0x3400))
_UART* pu;

#define LSR_THRE    (1<<5)  /* Transmitter Holding Register Empty -     */
                            /*  : 1 */
                            /* ,  UART      . */
                            /* ,    THR     . */
                            /*      THREI,   . */
                            /*        THR. */
                            /*   FIFO   ,  XMIT FIFO ,  , */
                            /*   XMIT FIFO     . */
#define LSR_TEMT    (1<<6)  /* Transmitter Empty -   */
                            /*  : 1 */
                            /* ,   THR  TSR .   , */
                            /*       THR  TSR  . */
                            /*   FIFO   ,      XMIT FIFO,     TSR. */
#define LCR_DLAB    (1<<7)  /* Divisor Latch Access Bit */
                            /*  : 0 */
                            /*    : */
                            /*  0      RBR,THR,IER; */
                            /*  1      DLL,DLM */

enum LCR_WLS_BITS {
	LCR_WLS_5,
	LCR_WLS_6,
	LCR_WLS_7,
	LCR_WLS_8,
};

#define LCR_WLS(n)  ((n&3)<<0)  /* Word Length Select */
                                /*  : 00 */
                                /*      : */
                                /*  00 - 5  */
                                /*  01 - 6  */ 
                                /*  10 - 7  */ 
                                /*  11 - 8  */

typedef enum {
    CMD_10=0,
    CMD_20,
    CMD_30,
    CMD_40,
    CMD_50,
    CMD_55,
    CMD_60,
    CMD_80,
    CMD_88,
    CMD_90,
    CMD_98,
    CMD_A0,
    CMD_AA,
    CMD_D0,
    CMD_F0,
    CMD_FF,
} FLASH_CMD;

typedef struct {
    unsigned int type;
    unsigned int base;
    unsigned int addr_odd;
	unsigned int addr_even;
    unsigned int cmd_10;
    unsigned int cmd_20;
    unsigned int cmd_30;
    unsigned int cmd_40;
    unsigned int cmd_50;
	unsigned int cmd_55;
    unsigned int cmd_60;
    unsigned int cmd_80;
	unsigned int cmd_88;
	unsigned int cmd_90;
    unsigned int cmd_98;
    unsigned int cmd_a0;
    unsigned int cmd_aa;
    unsigned int cmd_d0;
	unsigned int cmd_f0;
    unsigned int cmd_ff;
} FLASH_DESCRIPTOR;

#define FLASH16(n)  ((n&0xff)|((n&0xff)<<16))
#define ADR32(n)    ((n)<<2)

//    
#define ADDR_DUMMY  0xa0000000

#define BASE_ADDR			0xbf000000

#define MAN_ID_OFFSET       0x0
#define DEVICE_ID_OFFSET    0x1

#define HDR                 4

#define WSMS                0x00800080

extern int vsnprint(char *buf, size_t size, const char *fmt, va_list args);

#endif

